--
-- Cosmology.hs
--
-- The functions which determine the cosmological model.
--
-- Gregory Wright, 19 June 2006
--


module Cosmology (
        -- * The logarithmic derivative of the scale factor: (1 / R) * (dR / dt)
        scaleFactorDot
) where


import Thermal

newtonianG = 6.6742e-08  -- the newtonian gravitational constant,
                         -- units cm^3 g^{-1} s^{-2}.


-- | just the Friedmann model, for now.
--
scaleFactorDot :: Double    -- ^ time
               -> Double    -- ^ temperature / 10^9 K
               -> Double    -- ^ density / (T / 10^9 K)^3, units g cm^{_3}
               -> Double    -- ^ normalized electron chemical potential
               -> Double
scaleFactorDot t t9 hv phi =
        let
                rhoTotal = totalEnergyDensity t9 hv phi
        in
                sqrt ((8.0 / 3.0) * pi * newtonianG * rhoTotal)
