--
-- Initialize.hs, initialize the nucleosynthesis integration
--

module Initialize (
        nNuclides,
        t9electron,
        t9Initial,
        t9small,
        charDensity,
        hvInitial,
        deltaNPmass,
        neutrinoSpecies,

        initialConditions,
) where


import Data.Packed.Vector

import Utils


nNuclides  = 26

nCosmicParams = 3
neutrinoSpecies = 3

t9small = 0.05   -- ^ Below 10^8 K, we can ignore the electron and positron
                 --   contribution to just about anything.

t9electron  = 5.930    -- ^ The electron mass, in units of 10^9 K, k / (m_e * c^2)
charDensity = 3205.7   -- ^ characteristic density, (2 / pi^2) * m_e * (m_e * c / hbar)^3  g cm^{-3}
deltaNPmass = 2.531    -- ^ (m_N - m_P) / m_e

eta = 3.162e-10  -- initial photon to baryon ratio

t9Initial  = 100.0
hvInitial  = 3.3683e+04 * 2.75 * eta
rhob0 = hvInitial * (t9Initial^3)


initialConditions :: Vector Double
initialConditions = 
        fromList (initialCosmic ++ initialAbundances)


initialCosmic :: [Double]
initialCosmic =
        let
                t9  = t9Initial
                hv  = hvInitial

                z = t9electron / t9
                p z n = (-1)^(n + 1) * (fromIntegral n) * bigL((fromIntegral n) * z)
                phi = hv * (1.784e-05 * yp) / (0.5 * (z^3) * sumSeries p 1 5 z)

                yp = 1.0 / (exp(-15.011 / t9) + 1.0)
        in
                [t9, hv, phi]


initialAbundances :: [Double]
initialAbundances =
        let
                t9  = t9Initial

                yn = 1.0 / (exp( 15.011 / t9) + 1.0)
                yp = 1.0 / (exp(-15.011 / t9) + 1.0)
                yd = yn * yp * rhob0 * exp(25.82 / t9) / (0.471e+10 * (t9 ** 1.5))

                initNuclides = [yn, yp, yd]
        in
                initNuclides ++ (replicate (nNuclides - length initNuclides) 0.0)
