{-# INCLUDE <gsl/gsl_sf_bessel.h> #-}
--
-- Utils.hs, utility functions for computing the chemical potential
--


module Utils where


import Numeric.GSL.Special.Bessel


-- | Functions which appear in the formulas for the electron plus positron
--   energy density and pressure.  See Fowler and Hoyle, Ap. J. Suppl. No. 91, 9,
--   201 (1964).
--
bigL :: Double
     -> Double
bigL z = if z > 0.0
             then if z < 500.0
                      then (bessel_Kn 2 z) / z
                      else 0.0
             else error ("negative z in bigL : " ++ (show z))


bigM :: Double
     -> String
     -> Double
bigM z str = if z > 0.0
             then if z < 500.0
                      then (1 / (4.0 * z)) * (3.0 * (bessel_Kn 3 z) + (bessel_K1 z))
                      else 0.0
             else error ("negative z in bigM : " ++ (show z) ++ " from " ++ str)

bigN :: Double
     -> Double
bigN z = if z > 0.0
             then if z < 500.0
                      then (1 / (2.0 * z)) * ((bessel_Kn 4 z) + (bessel_Kn 2 z))
                      else 0.0
             else error ("negative z in bigN : " ++ (show z))


-- | A helper function to sum series.
-- 
--   Given a function for the n-th term of in the series, f n x, sumSeries sums
--   the terms from n = nmin to n = nmax.  It is expressed in terms of sumSeries'
--   whose definition is tail recursive, and uses `seq` to ensure that all
--   arguments are evaluated (i.e., passed by value).
--
sumSeries :: Num b => (a -> Int -> b) -> Int -> Int -> a -> b
sumSeries f min max x = sumSeries' f min max x 0
        where
                sumSeries' f min max x acc
                        | min `seq` max `seq` acc `seq` x `seq` False = undefined
                        | min > max    = acc
                        | otherwise    = sumSeries' f (min + 1) max x (acc + f x min)

